@@ -43,7 +43,7 @@ from enum import Enum
 from vantage6.common.docker.addons import (
     ContainerKillListener, check_docker_running, running_in_docker
 )
-from vantage6.common.globals import VPN_CONFIG_FILE
+from vantage6.common.globals import VPN_CONFIG_FILE, PING_INTERVAL_SECONDS
 from vantage6.common.exceptions import AuthenticationException
 from vantage6.common.docker.network_manager import NetworkManager
 from vantage6.common.task_status import TaskStatus
@@ -298,10 +298,17 @@ class Node(object):
             database=task.get('database', 'default')
         )
 
-        # save task status to the server and send socket event to update others
+        # save task status to the server
+        update = {'status': task_status}
+        if task_status == TaskStatus.NOT_ALLOWED:
+            # set finished_at to now, so that the task is not picked up again
+            # (as the task is not started at all, unlike other crashes, it will
+            # never finish and hence not be set to finished)
+            update['finished_at'] = datetime.datetime.now().isoformat()
         self.server_io.patch_results(
-            id=taskresult['id'], result={'status': task_status}
+            id=taskresult['id'], result=update
         )
+        # send socket event to alert everyone of task status change
         self.socketIO.emit(
             'algorithm_status_change',
             data={
@@ -486,6 +493,9 @@ class Node(object):
             self.log.critical('Unable to authenticate. Exiting')
             exit(1)
 
+        # start thread to keep the connection alive by refreshing the token
+        self.server_io.auto_refresh_token()
+
     def private_key_filename(self) -> Path:
         """Get the path to the private key."""
 
@@ -788,6 +798,23 @@ class Node(object):
         self.log.info(f'Connected to host={self.server_io.host} on port='
                       f'{self.server_io.port}')
 
+        self.log.debug("Starting thread for to ping the server to notify this"
+                       " node is online.")
+        self.socketIO.start_background_task(self.__socket_ping_worker)
+
+    def __socket_ping_worker(self) -> None:
+        """
+        Send ping messages periodically to the server over the socketIO
+        connection to notify the server that this node is online
+        """
+        while True:
+            try:
+                self.socketIO.emit('ping', namespace='/tasks')
+            except Exception:
+                self.log.exception('Ping thread had an exception')
+            # Wait before sending next ping
+            time.sleep(PING_INTERVAL_SECONDS)
+
     def get_task_and_add_to_queue(self, task_id: int) -> None:
         """
         Fetches (open) task with task_id from the server. The `task_id` is
@@ -886,6 +913,48 @@ class Node(object):
             )
         return killed_algos
 
+    def share_node_details(self) -> None:
+        """
+        Share part of the node's configuration with the server.
+
+        This helps the other parties in a collaboration to see e.g. which
+        algorithms they are allowed to run on this node.
+        """
+        # check if node allows to share node details, otherwise return
+        if not self.config.get('share_config', True):
+            self.log.debug("Not sharing node configuration in accordance with "
+                           "the configuration setting.")
+            return
+
+        config_to_share = {}
+
+        encryption_config = self.config.get('encryption')
+        if encryption_config:
+            if encryption_config.get('enabled') is not None:
+                config_to_share['encryption'] = \
+                    encryption_config.get('enabled')
+
+        # TODO v4+ remove the old 'allowed_images' key, it's now inside
+        # 'policies'. It's now overwritten below if 'policies' is set.
+        allowed_algos = self.config.get('allowed_images')
+        config_to_share['allowed_algorithms'] = allowed_algos \
+            if allowed_algos else 'all'
+
+        # share node policies (e.g. who can run which algorithms)
+        policies = self.config.get('policies', {})
+        config_to_share['allowed_algorithms'] = \
+            policies.get('allowed_algorithms', 'all')
+        if policies.get('allowed_users') is not None:
+            config_to_share['allowed_users'] = policies.get('allowed_users')
+        if policies.get('allowed_organizations') is not None:
+            config_to_share['allowed_orgs'] = \
+                policies.get('allowed_organizations')
+
+        self.log.debug(f"Sharing node configuration: {config_to_share}")
+        self.socketIO.emit(
+            'node_info_update', config_to_share, namespace='/tasks'
+        )
+
     def cleanup(self) -> None:
 
         if hasattr(self, 'socketIO') and self.socketIO:
